<?php
namespace Estoque\Repositories;

class Inverota extends \Doctrine\ORM\EntityRepository
{


    CONST STATUS_N = "N";


    /**
     * Verifica se há inventário rotativo pendente.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtproc
     *
     * @return mixed
     */
    public function consultaInventPendenteRota($nrorg, $filial, $dtproc){
        $sql = "SELECT I.*
                  FROM INVEROTA I
                 WHERE I.NRORG      = :P_NRORG
                   AND I.CDFILIAL   = :P_CDFILIAL
                   AND I.DTINVEROTA = (SELECT MAX(DTINVEROTA)
                                        FROM INVEROTA
                                       WHERE I.NRORG      = :P_NRORG
                                         AND CDFILIAL   = :P_CDFILIAL
                                         AND DTINVEROTA < :P_DTINVEROTA)
                   AND I.IDINVENTOK = :P_STATUS_N";

        $params = array(
            "P_CDFILIAL"   => $filial,
            "P_NRORG"      => $nrorg,
            "P_STATUS_N"   => self::STATUS_N,
            "P_DTINVEROTA" => $dtproc,
        );

        $entityManager = $this->getEntityManager();
        $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
        $resultSetMapping->addRootEntityFromClassMetadata('Estoque\Entities\Inverota', "I");

        $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);
        return count($result)>0 ? $result : false;
    }


    public function insereInventarioRotativoLocal(
        $nrorg, $filial, $dtInventario, $qtdEstoque, $qtdInvent, $almox, $local, $produto, $operador, $dataAtual
    ){
        $em = \Setforms2\Shared\Registry::getEntityManager();

        $sql = "INSERT INTO INVEROTA (CDFILIAL, NRORG, DTINVEROTA, CDPRODUTO, QTINVEROTA,
                                      QTESTOQUE, CDALMOXARIFE, CDLOCALESTOQ, CDOPERINCLUSAO,
                                      DTINCLUSAO)
                            (SELECT :P_CDFILIAL, :P_NRORG, :P_DTINVEROTA, :P_CDPRODUTO, :P_QTINVEROTA,
                                    :P_QTESTOQUE, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_CDOPERINCLUSAO,
                                    :P_DTINCLUSAO
                               FROM DUAL
                               WHERE NOT EXISTS (SELECT 1
                               FROM INVEROTA
                              WHERE CDFILIAL  = :P_CDFILIAL
                                AND NRORG     = :P_NRORG
                                AND CDPRODUTO = :P_CDPRODUTO
                                AND CDALMOXARIFE = :P_CDALMOXARIFE
                                AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                                AND DTINVEROTA   = :P_DTINVEROTA))";

        $params = array(
            "P_NRORG"        => $nrorg,
            "P_CDFILIAL"     => $filial,
            "P_DTINVEROTA"   => $dtInventario,
            "P_CDPRODUTO"    => $produto,
            "P_QTESTOQUE"   => $qtdEstoque,
            "P_QTINVEROTA"   => $qtdInvent,
            "P_CDALMOXARIFE" => $almox,
            "P_CDLOCALESTOQ" => $local,
            "P_CDOPERINCLUSAO" => $operador,
            "P_DTINCLUSAO" => $dataAtual,
        );

        return $em->getConnection()->executeQuery($sql, $params);
    }

    /**
     * Verifica status do invetario.
     *
     * @param $cdfilial     string
     * @param $cdalmoxarife string
     * @param $cdlocalestoq string
     * @param $dtinverota   string
     *
     * @return array
     */
    public function inventarioOk ($cdfilial, $cdalmoxarife, $cdlocalestoq, $dtinverota) {

        $sql =
            "SELECT IDINVENTOK
               FROM INVEROTA
              WHERE CDFILIAL = :P_CDFILIAL
                AND CDALMOXARIFE = :P_CDALMOXARIFE
                AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                AND IDINVENTOK = 'N'
                AND DTINVEROTA = (SELECT MAX(DTINVEROTA)
                                    FROM INVEROTA
                                   WHERE CDFILIAL = :P_CDFILIAL
                                     AND CDALMOXARIFE = :P_CDALMOXARIFE
                                     AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                                     AND DTINVEROTA < :P_DTINVEROTA
                                )";
        $params = array(
            'P_CDFILIAL'     => $cdfilial,
            'P_CDALMOXARIFE' => $cdalmoxarife,
            'P_CDLOCALESTOQ' => $cdlocalestoq,
            'P_DTINVEROTA'   => $dtinverota
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
    }

    public function verificaInventarioPendente($filial, $almoxarifado, $localEstoque, $dtInverota, $organizacao)
    {
        $sql = "SELECT IDINVENTOK,
				  CDALMOXARIFE,
				  CDLOCALESTOQ
				FROM INVEROTA
				WHERE (CDFILIAL = :P_CDFILIAL)
				AND (NRORG = :P_NRORG)
				AND (DTINVEROTA =
				  (SELECT MAX(DTINVEROTA)
				  FROM INVEROTA
				  WHERE (CDFILIAL   = :P_CDFILIAL)
				  AND (CDALMOXARIFE = :P_CDALMOXARIFE)
				  AND (CDLOCALESTOQ = :P_CDLOCALESTOQ)
				  AND (DTINVEROTA   < :P_DTINVEROTA)
				  ))
				AND (CDALMOXARIFE = :P_CDALMOXARIFE)
				AND (CDLOCALESTOQ = :P_CDLOCALESTOQ)
				AND (IDINVENTOK   = 'N')
				UNION ALL
				SELECT IDINVLOTEOK,
				  CDALMOXARIFE,
				  CDLOCALESTOQ
				FROM INVEROTALOTE
				WHERE (CDFILIAL = :P_CDFILIAL)
				AND (DTINVELOTE =
				  (SELECT MAX(DTINVELOTE)
				  FROM INVEROTALOTE
				  WHERE (CDFILIAL   = :P_CDFILIAL)
				  AND (CDALMOXARIFE = :P_CDALMOXARIFE)
				  AND (CDLOCALESTOQ = :P_CDLOCALESTOQ)
				  AND (DTINVELOTE   < :P_DTINVEROTA)
				  ))
				AND (CDALMOXARIFE = :P_CDALMOXARIFE)
				AND (CDLOCALESTOQ = :P_CDLOCALESTOQ)
				AND (IDINVLOTEOK  = 'N')";

        $params = array(
            "P_CDFILIAL" 	=> $filial,
            "P_CDALMOXARIFE"=> $almoxarifado,
            "P_CDLOCALESTOQ"=> $localEstoque,
            "P_DTINVEROTA"  => $dtInverota,
            "P_NRORG"       => $organizacao,
        );

        return \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
    }


}