<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 15/12/2014
 * Time: 08:53
 */

namespace Folha\Logic\ProgramacaoOcorrencia;

use Folha\Entities\PtoTipoprogocorrencia;
use Geral\Util\NovoCodigo;
use Setforms2\Shared\Registry;

class TipoProgramacao {
    public static function insertOrUpdateTipoProgramacao($dados)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try {
            $nrOrgReturn = $dados['NRORG_AMBIENTE'];

            if($dados['NRORG_AMBIENTE'] != $dados['NRORG_PADRAO'] && $dados['NRORG_QUERY'] == $dados['NRORG_PADRAO']) {
                //Caso esteja alterando uma de trabalho mas os dados sejam os mesmos de uma padrão, exclui a de trabalho e mantém a padrão.
                $tipoProgPadr = PtoTipoprogocorrencia::findOneByNrordemprogocorrenciaAndIdprogocoraconfirmarAndCdtipoprogocorrenciaAndNrorg($dados['NRORDEMPROGOCORRENCIA'],
                                                                                                                                           $dados['IDPROGOCORACONFIRMAR'],
                                                                                                                                           $dados['CDTIPOPROGOCORRENCIA'],
                                                                                                                                           $dados['NRORG_PADRAO']);

                if(is_object($tipoProgPadr)){
                    $nrOrgReturn = $dados['NRORG_QUERY'];
                    return array($dados['NRTIPOPROGOCORRENCIA'], $nrOrgReturn);
                }

                $dados['NRTIPOPROGOCORRENCIA'] = NovoCodigo::geraCodigo($dados['NRORG_PADRAO'], 'PTO_TIPOPROGOCORRENCIA', 1);
                $tipoProgOcorr = new PtoTipoprogocorrencia();
                $tipoProgOcorr->setDtinclusao($dados['DATAATUAL']);
                $tipoProgOcorr->setNrorginclusao($dados['NRORG_AMBIENTE']);
                $tipoProgOcorr->setCdoperinclusao($dados['CDOPERADOR']);

                self::setColumsTipoProgramacao($tipoProgOcorr, $dados);
            } else {
                //Caso esteja alterando uma de trabalho mas os dados sejam os mesmos de uma padrão, exclui a de trabalho e mantém a padrão.
                $tipoProgPadr = PtoTipoprogocorrencia::findOneByNrordemprogocorrenciaAndIdprogocoraconfirmarAndCdtipoprogocorrenciaAndNrorg($dados['NRORDEMPROGOCORRENCIA'],
                                                                                                                                           $dados['IDPROGOCORACONFIRMAR'],
                                                                                                                                           $dados['CDTIPOPROGOCORRENCIA'],
                                                                                                                                           $dados['NRORG_PADRAO']);

                if(is_object($tipoProgPadr) && $dados['NRORG_AMBIENTE'] != $dados['NRORG_PADRAO']) {
                    self::deleteTipoProgramacao($dados['NRTIPOPROGOCORRENCIA'], $dados['NRORG_AMBIENTE']);

                    $nrOrgReturn = $dados['NRORG_PADRAO'];
                } else {
                    //Em outros casos, apenas altera o necessário.
                    $tipoProgOcorr = PtoTipoprogocorrencia::findOneByNrtipoprogocorrenciaAndNrorg($dados['NRTIPOPROGOCORRENCIA'], $dados['NRORG_AMBIENTE']);

                    self::setColumsTipoProgramacao($tipoProgOcorr, $dados);
                }
            }

            $entityManager->flush();
            $db->commit();

            return array($dados['NRTIPOPROGOCORRENCIA'], $nrOrgReturn);
        } catch (\Exception $e) {
            $db->rollback();
            throw new \Exception($e->getMessage());
        }
    }

    private static function setColumsTipoProgramacao(PtoTipoprogocorrencia $tipoProgOcorr, $dados)
    {
        $tipoProgOcorr->setNrordemprogocorrencia($dados['NRORDEMPROGOCORRENCIA']);
        $tipoProgOcorr->setNrtipoprogocorrencia($dados['NRTIPOPROGOCORRENCIA']);
        $tipoProgOcorr->setCdtipoprogocorrencia($dados['CDTIPOPROGOCORRENCIA']);
        $tipoProgOcorr->setNmtipoprogocorrencia($dados['NMTIPOPROGOCORRENCIA']);
        $tipoProgOcorr->setIdprogocoraconfirmar($dados['IDPROGOCORACONFIRMAR']);
        $tipoProgOcorr->setDsobservacao($dados['DSOBSERVACAO']);
        $tipoProgOcorr->setNrorg($dados['NRORG_AMBIENTE']);
        $tipoProgOcorr->setNrorgultatu($dados['NRORG_AMBIENTE']);
        $tipoProgOcorr->setIdativo($dados['IDATIVO']);
        $tipoProgOcorr->setDtultatu($dados['DATAATUAL']);
        $tipoProgOcorr->setCdoperultatu($dados['CDOPERADOR']);
        $tipoProgOcorr->persist();
    }

    public static function deleteTipoProgramacao($nrTpProg, $nrOrg)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try {
            $tipoProgOcorr = PtoTipoprogocorrencia::findOneByNrtipoprogocorrenciaAndNrorg($nrTpProg, $nrOrg);
            $tipoProgPadr  = PtoTipoprogocorrencia::findOneByCdtipoprogocorrenciaAndNrorg($tipoProgOcorr->getCdtipoprogocorrencia(),
                                                                                          \FPA::getNrorgPadrao());

            $nrTpProgPadr = $tipoProgPadr->getNrtipoprogocorrencia();
            $nrOrdemPadr  = $tipoProgPadr->getNrordemprogocorrencia();
            $idProgConf   = $tipoProgPadr->getIdprogocoraconfirmar();

            $arrayReturn = array($nrTpProgPadr, $nrOrdemPadr, $idProgConf);

            if(is_object($tipoProgOcorr)) {
                $tipoProgOcorr->delete();
            }

            $entityManager->flush();
            $db->commit();

            return $arrayReturn;
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }
}

